import 'dart:async';

import 'package:args/command_runner.dart';

import 'package:daily_programmer_client_dart/app/command/challenge.dart';
import 'package:daily_programmer_client_dart/app/command/project.dart';
import 'package:daily_programmer_client_dart/app/command/run.dart';
import 'package:daily_programmer_client_dart/util/app_logger.dart';

/// Base of [CommandRunner] for set the verbose option.
class DailyProgramingCommanderRunner extends CommandRunner {
  static const _verboseOption = 'verbose';

  /// Init with verbose option support for all app.
  DailyProgramingCommanderRunner()
      : super('daily_programmer_dart_cli',
            'DailyPrograming is a terminal client for manage challenges.') {
    argParser.addFlag(_verboseOption,
        abbr: 'v',
        help: 'Output extra logging information.',
        defaultsTo: false);

    addCommand(new ChallengeCommand());
    addCommand(new ProjectCommand());
    addCommand(new RunCommand());
  }

  @override
  Future run(Iterable<String> args) async {
    final option = super.parse(args);
    AppLogger.log.isVerbose = option[_verboseOption];

    await runCommand(option);
  }
}
