import 'dart:async';

import 'package:daily_programmer_client_dart/util/entity_name.dart';
import 'package:path/path.dart' as p;

import 'package:daily_programmer_client_dart/generator/generator.dart';

/// Generator for challenge.
class ChallengeGenerator extends Generator {
  static const _templateFolder = 'challenge';
  static const List<String> _templateFileNames = const [
    'challenge.dart.mustache',
    'CHALLENGE.md.mustache',
  ];

  /// Class name of this challenge.
  final String className;

  /// Separator.
  final String selector;

  /// Challenge file name without extension.
  final String targetName;

  /// Dry run flag set from cli
  final bool dryRun;

  /// Return an instance of [ChallengeGenerator] with the context of parent of
  /// command.
  factory ChallengeGenerator(
          EntityName classEntityName, String destinationFolder,
          {bool dryRun: false}) =>
      new ChallengeGenerator._(
          classEntityName.camelCased,
          classEntityName.dashed,
          classEntityName.underscored,
          destinationFolder,
          dryRun);

  ChallengeGenerator._(this.className, this.selector, this.targetName,
      String destinationFolder, this.dryRun)
      : super(destinationFolder);

  // Gets a map from template file name to target file name.
  Map<String, String> _getTemplateTargetPaths() {
    final results = <String, String>{};
    for (final templateFileName in _templateFileNames) {
      results[p.join(_templateFolder, templateFileName)] =
          '$targetName.${templateFileName.split('.')[1]}';
    }

    return results;
  }

  @override
  Future generate() async {
    await renderAndWriteTemplates(_getTemplateTargetPaths(), dryRun: dryRun);
  }
}
