import 'package:test/test.dart';

import 'package:daily_programmer_client_dart/util/ansicolor.dart';

void main() {
  tearDown(() {
    colorDisabled = false;
  });

  test('foreground', () {
    final color = new Ansi()..rgb(r: 1.0, g: 0.8, b: 0.2);
    expect(color.write("Test Text"), "\x1B[38;5;221mTest Text\x1B[0m");
  });

  test('background', () {
    final color = new Ansi()..rgb(r: 0.4, g: 0.8, b: 1.0, bg: true);
    expect(color.write("Test Text"), "\x1B[48;5;117mTest Text\x1B[0m");
  });

  test('foreground and background', () {
    final color = new Ansi()
      ..rgb(r: 1.0, g: 0.8, b: 0.2)
      ..rgb(r: 0.4, g: 0.8, b: 1.0, bg: true);
    expect(color.write("Test Text"),
        "\x1B[38;5;221m\x1B[48;5;117mTest Text\x1B[0m");
  });

  test('foreground and background w/ resets', () {
    final color = new Ansi()
      ..rgb(r: 1.0, g: 0.8, b: 0.2)
      ..rgb(r: 0.4, g: 0.8, b: 1.0, bg: true);
    expect(color.write("Test${resetBackground()} Text${resetForeground()}Test"),
        "\x1B[38;5;221m\x1B[48;5;117mTest\x1B[49m Text\x1B[39mTest\x1B[0m");
  });

  test('direct xterm', () {
    final color = new Ansi()..xterm(200)..xterm(100, bg: true);
    expect(color.write("Test Text"),
        "\x1B[38;5;200m\x1B[48;5;100mTest Text\x1B[0m");
  });

  test('call() == write()', () {
    final color = new Ansi()
      ..rgb(r: 1.0, g: 0.8, b: 0.2)
      ..rgb(r: 0.4, g: 0.8, b: 1.0, bg: true);
    expect(color.write("Test Text"), color("Test Text"));
  });

  test('interpolated == write()', () {
    final color = new Ansi()
      ..rgb(r: 1.0, g: 0.8, b: 0.2)
      ..rgb(r: 0.4, g: 0.8, b: 1.0, bg: true);
    expect("${color}Test Text${color.up}", color("Test Text"));
  });

  test('system colors', () {
    final color = new Ansi();
    expect((color..black()).down, "\x1B[38;5;0m");
    expect((color..red()).down, "\x1B[38;5;1m");
    expect((color..green()).down, "\x1B[38;5;2m");
    expect((color..yellow()).down, "\x1B[38;5;3m");
    expect((color..blue()).down, "\x1B[38;5;4m");
    expect((color..magenta()).down, "\x1B[38;5;5m");
    expect((color..cyan()).down, "\x1B[38;5;6m");
    expect((color..white()).down, "\x1B[38;5;7m");

    expect((color..black(bold: true)).down, "\x1B[38;5;8m");
    expect((color..red(bold: true)).down, "\x1B[38;5;9m");
    expect((color..green(bold: true)).down, "\x1B[38;5;10m");
    expect((color..yellow(bold: true)).down, "\x1B[38;5;11m");
    expect((color..blue(bold: true)).down, "\x1B[38;5;12m");
    expect((color..magenta(bold: true)).down, "\x1B[38;5;13m");
    expect((color..cyan(bold: true)).down, "\x1B[38;5;14m");
    expect((color..white(bold: true)).down, "\x1B[38;5;15m");

    expect((color..reset()).down, "");

    expect((color..black(bg: true)).down, "\x1B[48;5;0m");
    expect((color..red(bg: true)).down, "\x1B[48;5;1m");
    expect((color..green(bg: true)).down, "\x1B[48;5;2m");
    expect((color..yellow(bg: true)).down, "\x1B[48;5;3m");
    expect((color..blue(bg: true)).down, "\x1B[48;5;4m");
    expect((color..magenta(bg: true)).down, "\x1B[48;5;5m");
    expect((color..cyan(bg: true)).down, "\x1B[48;5;6m");
    expect((color..white(bg: true)).down, "\x1B[48;5;7m");

    expect((color..black(bg: true, bold: true)).down, "\x1B[48;5;8m");
    expect((color..red(bg: true, bold: true)).down, "\x1B[48;5;9m");
    expect((color..green(bg: true, bold: true)).down, "\x1B[48;5;10m");
    expect((color..yellow(bg: true, bold: true)).down, "\x1B[48;5;11m");
    expect((color..blue(bg: true, bold: true)).down, "\x1B[48;5;12m");
    expect((color..magenta(bg: true, bold: true)).down, "\x1B[48;5;13m");
    expect((color..cyan(bg: true, bold: true)).down, "\x1B[48;5;14m");
    expect((color..white(bg: true, bold: true)).down, "\x1B[48;5;15m");
  });

  test('rgb overflow', () {
    final color = new Ansi()
      ..rgb(r: 2.0, g: 2.8, b: 2.2)
      ..rgb(r: 2.4, g: 2.8, b: 2.0, bg: true);
    expect(color.write("Test Text"),
        "\x1B[38;5;231m\x1B[48;5;231mTest Text\x1B[0m");
  });

  test('rgb underflow', () {
    final color = new Ansi()
      ..rgb(r: -1.0, g: -2.8, b: -2.2)
      ..rgb(r: -1.0, g: -2.8, b: -2.0, bg: true);
    expect(
        color.write("Test Text"), "\x1B[38;5;16m\x1B[48;5;16mTest Text\x1B[0m");
  });

  test('grayscale', () {
    final color = new Ansi();

    for (var i = 0; i < 24; i++) {
      expect((color..gray(level: i / 23)).down, "\x1B[38;5;${232 + i}m",
          reason: "fg failed at $i");
    }

    expect((color..reset()).down, "");

    for (var i = 0; i < 24; i++) {
      expect(
          (color..gray(level: i / 23, bg: true)).down, "\x1B[48;5;${232 + i}m",
          reason: "bg failed at $i");
    }
  });

  test('colorDisabled', () {
    colorDisabled = false;
    final color = new Ansi()
      ..rgb(r: 1.0, g: 0.8, b: 0.2)
      ..rgb(r: 0.4, g: 0.8, b: 1.0, bg: true);
    colorDisabled = true;
    expect(color.write("Test Text"), "Test Text");
  });
}
